@@ -5,38 +5,23 @@ pseudotime and terminal state probabilities for each cell. Then, applies
 MIRA's lineage tree inference algorithm to reconstruct bicurcating tree
 structure of differentation data.
 
-Examples
---------
->>> sc.tl.diffmap(adata)
->>> mira.pl.plot_eigengap(adata)
->>> sc.pp.neighbors(adata, use_rep = 'X_diffmap', key_added = 'X_diffmap', n_neighbors = 30, n_pcs = 3)
->>> mira.time.normalize_diffmap(adata)
->>> mira.get_connected_components(adata)
->>> mira.time.get_transport_map(adata, start_cell = 0)
->>> mira.time.find_terminal_cells(adata)
-[10,20,30]
->>> mira.time.get_branch_probabilities(adata, terminal_cells = {
-    "lineage1" : 10, "lineage2" : 20, "lineage3" : 3
-})
->>> mira.time.get_tree_structure(adata, treshold = 0.5)
+Notably, these algorithms exclusively utilize the nearest-neighbor of cells,
+which eliminates biases, distortions, and compression of complex 
+topologies replete in UMAP-based pseudotime trajectory inference algorithms.
 '''
 
-from networkx.algorithms import components
 import numpy as np
 import networkx as nx
 from scipy import sparse
 from scipy.sparse.base import isspmatrix
-from sklearn.neighbors import NearestNeighbors
-from numpy.linalg import inv, pinv
-from sklearn.linear_model import LogisticRegression
+from numpy.linalg import inv
 from joblib import Parallel, delayed
 from scipy.sparse import csgraph
-from scipy.stats import entropy, pearsonr, norm
+from scipy.stats import entropy, pearsonr
 from copy import deepcopy
 from scipy.sparse.linalg import eigs
 import logging
-import tqdm
-from tqdm.std import trange
+from tqdm.auto import tqdm, trange
 import mira.adata_interface.core as adi
 import mira.adata_interface.pseudotime as pti
 from functools import partial
@@ -108,7 +93,7 @@ def normalize_diffmap(rescale = True,*,diffmap, eig_vals):
     components with larger eigenvalues. 
 
     Rescaling eigenvectors produces a smoother pseudotime representation,
-    but may over-smooth the nearest neighbor space and may distort branch
+    but may over-smooth the nearest neighbor space and distort branch
     points and lineage paths. If branch points appear poorly-defined, try
     setting `rescale` to False.
 
@@ -290,6 +275,8 @@ def get_pseudotime(max_iterations = 25, n_waypoints = 3000, n_jobs = 1,*, start_
 
         if converged:
             break
+    
+    t.reset(t.n)
 
     pseudotime = pseudotime - pseudotime.min() #make 0 minimum
     waypoint_weights = W
@@ -389,7 +376,7 @@ def get_transport_map(ka = 5, n_jobs = 1,*, start_cell = None, distance_matrix,
             Pseudotime of cells
         `.obsp["transport_map"]` : scipy.spmatrix[float] of shape (n_cells, n_cells)
             Sparse matrix of transition probabilities between cells.
-        `.uns["iroot"]` : str
+        `.uns["start_cell"]` : str
             name/id of start cell
     '''
     
@@ -414,7 +401,8 @@ def get_transport_map(ka = 5, n_jobs = 1,*, start_cell = None, distance_matrix,
 
 
 @adi.wraps_functional(pti.fetch_transport_map, pti.get_cell_ids, ['transport_map'])
-def find_terminal_cells(iterations = 1, max_termini = 15, threshold = 1e-3, *, transport_map):
+def find_terminal_cells(iterations = 1, max_termini = 15, threshold = 1e-3, 
+        seed = None, *, transport_map):
     '''
     Uses transport map to identify terminal cells where differentiation progress
     reaches a steady state. Results are stochastic based on SVD initialization.
@@ -434,19 +422,49 @@ def find_terminal_cells(iterations = 1, max_termini = 15, threshold = 1e-3, *, t
         Treshold includes eigenvectors that are greater than 1 - `threshold`.
         To loosen the definition of terminal states and identify more cells,
         increase `threshold`, e.g. 1e-2.
+    seed : int, default = None
+        Seed for terminal state calling. Seed initializes SVD decomposition
+        of transport map.
 
     Returns
     -------
     terminal_cells : list[str]
         List of terminal cell names
+
+    Examples
+    --------
+
+    First, use `find_terminal_cells` to create a list of possible terminal states
+    taken from the transport map.
+
+    .. code-block:: python
+        
+        >>> terminal_cells = mira.time.find_terminal_cells(data, threshold = 1e-2)
+
+    To elimnate spurious terminal states or to subset to *apriori* known 
+    termini, plot them like so:
+
+    .. code-block:: python
+
+        >>> ax = sc.pl.umap(data, color = 'mira_pseudotime', show = False,
+        ...   **umap_kwargs, color_map = 'magma')
+        >>> sc.pl.umap(data[terminal_cells], na_color = 'black', show = False, ax = ax, 
+        ...   size = 200, title = 'Terminal Cells')
+
+    .. image:: /_static/pseudotime/mira.time.find_terminal_cells.png
+        :width: 400
+
     '''
 
+    np.random.seed(seed)
+
     assert(transport_map.shape[0] == transport_map.shape[1])
     assert(len(transport_map.shape) == 2)
 
     def _get_stationary_points():
 
-        vals, vectors = eigs(transport_map.T, k = max_termini)
+        v0 = np.random.randn((transport_map.shape[0]))
+        vals, vectors = eigs(transport_map.T, k = max_termini, v0 = v0)
 
         stationary_vecs = np.isclose(np.real(vals), 1., threshold)
 
@@ -495,11 +513,22 @@ def get_branch_probabilities(*, transport_map, terminal_cells):
 
     Examples
     --------
-    >>> mira.time.get_branch_probabilities(adata, terminal_cells = {
-        "A" : "AACACATGTGTAC", "B" : "GTGTGCGAGCGAATT"
-    })
-    >>> sc.pl.umap(adata, color = ["A_prob", "B_prob"])
-    >>> sc.pl.umap(adata, color = 'differentiation_entropy")
+
+    The user must provide a dictionary with keys as terminal state names and
+    values as the representative terminal cell. The cell may be indicated by
+    barcode/obs_name or index number.
+
+    .. code-block:: python
+
+        >>> mira.time.get_branch_probabilities(adata, terminal_cells = {
+        ...    "A" : "TATGCGCATCGCGCGC", "B" : "GCGTGGCATCGCGCGC"
+        ... })
+        >>> sc.pl.umap(data, color = [x + '_prob' for x in data.uns['lineage_names']], 
+        ... color_map='magma')
+
+    .. image:: /_static/pseudotime/mira.time.get_branch_probabilities.png
+        :width: 1200
+    
     '''
 
     logger.info('Simulating random walks ...')
@@ -559,6 +588,7 @@ def get_lineage_prob_fc(*, branch_probs, start_cell):
     ])
 
     return lineage_prob_fc
+    
 
 def get_lineage_branch_time(lineage1, lineage2, pseudotime, prob_fc, threshold = 0.5):
 
@@ -594,6 +624,12 @@ def get_tree_structure(threshold = 0.1,*, lineage_names, branch_probs, pseudotim
         thresholds are more sensitive to divergence events. Try multiple values
         for threshold to find best value to parse lineage tree. Typically, 0.1 is a
         very sensitive value, while 1.0-2.0 are very tolerant.
+    cellrank : boolean, default=False
+        If using cellrank to assign terminal states and lineage probabilities,
+        set this option to **True**.
+    start_cell : int or barcode, default=None
+        Cell representing start state of differentiation. *Only needed if 
+        cellrank is True*.
 
     Returns
     -------
@@ -604,22 +640,32 @@ def get_tree_structure(threshold = 0.1,*, lineage_names, branch_probs, pseudotim
                 into terminal lineages A and B, all cells before the branch point
                 would be labeled "A, B", while cells after the branch point would be
                 labeled "B" if they followed the path to the "B" terminal state. 
-            `.uns["connectivities_tree"] : np.ndarray[boolean] of shape (2*num_lineages - 1, 2*num_lineages - 1)
+            `.uns["connectivities_tree"] : np.ndarray[boolean] of shape 
+                (2*num_lineages - 1, 2*num_lineages - 1).
                 Digraph of relationships between `tree_states`. For instance,
                 an edge would be drawn from "A, B" to "A" and to "B".
 
-                                     "A"
-                                    /   
-                              "A, B"
-                                    \__"B"
+                .. code-block:: text
+                
+                            "A"
+                        /   
+                    "A, B"
+                        \__"B"
 
             `.uns["tree_state_names"] : np.ndarray[str] of shape (2*num_lineages - 1,)
                 Tree state labels for columns and rows of `connectivities_tree`
 
     Examples
     --------
-    >>> mira.time.get_tree_structure(adata, threshold = 0.5)
-    >>> sc.pl.umap(adata, color= "tree_states")
+
+    .. code-block:: python
+
+        >>> mira.time.get_tree_structure(data, threshold = 1)
+        >>> sc.pl.umap(data, color = 'tree_states', palette = 'Set2', 
+        ...   **umap_kwargs, title = '', legend_loc='on data')
+
+    .. image:: /_static/pseudotime/mira.time.get_tree_structure.png
+        :width: 400
 
     '''
     def get_all_leaves_from_node(edge):
@@ -629,9 +675,6 @@ def get_tree_structure(threshold = 0.1,*, lineage_names, branch_probs, pseudotim
         else:
             return [edge]
 
-    def get_node_name(self, node):
-        return ', '.join(map(str, self.lineage_names[self.get_all_leaves_from_node(node)]))
-
     def merge_rows(x, col1, col2):
         return np.hstack([
             (x[:,col1] + x[:, col2])[:, np.newaxis], #merge two lineages into superlineage
@@ -698,7 +741,7 @@ def get_tree_structure(threshold = 0.1,*, lineage_names, branch_probs, pseudotim
         if node == 'Root':
             return node
 
-        return ', '.join(set(get_all_leaves_from_node(node)))
+        return ', '.join(sorted(list(set(get_all_leaves_from_node(node)))))
             
     state_names = {
         edge[2]['state'] : get_node_name(edge[1])
