@@ -3,4 +3,4 @@ from mira.tools.chip_scan import get_ChIP_hits_in_peaks
 from mira.tools.motif_scan import get_motif_hits_in_peaks
 from mira.tools.lite_nite import get_NITE_score_cells, get_NITE_score_genes, get_chromatin_differential
 from mira.tools.tf_targeting import driver_TF_test
-from mira.tools.enrichr_enrichments import post_genelist, fetch_ontology, fetch_ontologies
\ No newline at end of file
+from mira.tools.enrichr_enrichments import post_genelist, fetch_ontology, fetch_ontologies, LEGACY_ONTOLOGIES
\ No newline at end of file
